//
//	ear_chords.cp
//
//	Contains subroutines for calculating things having to do with chords.
//	No input or output, just pure math.
//
//	The concept of a canonical chord, discussed in more detail below,
//	is that it is the class of all chords that are the same except for
//	voicing and absolute pitch
//	("Voicing" should include both shifting notes by octaves and doubling notes;
//	actually I haven't been careful about doublings -- should go back over code).
//
//	I have three data structures for storing chords:
//		(1) list of notes
//			Depending on context, an over-all shift in pitch
//			may or may not be significant.  Usually I use
//			0 as the lowest note, then when I'm ready to
//			actually play it I add an offset to them all.
//		(2) list of chord items
//			Items are defined in ear_defines.h.
//			Items may be basic chord types or alterations.
//			Basic chords are divided into triads and 7th chords.
//			Alterations can be added tones or omitted tones.
//			I sometimes refer to alterations in general as
//			added tones.
//			A well-constructed item list should
//				(1) contain exactly one basic chord
//				(2) not contain the same alteration twice
//			When an item list is passed to a subroutine,
//			I normally assume it has been constructed correctly.
//			It is the responsibility of the code creating the
//			item list to make sure it is constructed correctly.
//			Ordinarily the basic chord should be the first item,
//			but code should be able to cope if it's not.
//		(3) canonical chord number
//			For a given number of notes, I define canonical chord
//			numbers for all the different canonical chords.
//			The ordering is determined by "dissonance" and
//			"squishedness", defined below.
//		(4) quick label
//			A quick and dirty way of assigning a unique integer
//			label to a given chord.  Low-order 4 bits are
//			the 2nd note of the canonical voicing, etc.
//			(Don't need 1st note, it's always zero.)
//
// After coding everything up for canonical chord numbers, I didn't end
// up using the concept very much in the ear workout software.  In the
// chord workout, however, the criterion for a correct answer is that
// it should have the same canonical form as the chord the computer played.
//
//
// Highlights:
//
// make_canonical_voicing	Makes the canonical voicing of a given chord.
// describe_chord		Converts an item list to a string in guitar chord
//					notation.
// make_chord			Converts an item list to a list of notes.
// notes_to_items		Converts a list of notes to an item list.
// items_describe_same_chord	Tells whether 2 item lists describe the same
//					canonical chord.
// same_canonical_form		Compares 2 lists of notes according to my
//					ordering.
// remove_doublings		If any two notes are a unison or differ only by
//					octaves, eliminate the higher one.
//--------------------------------------------------------------------------------
/*
        

Lots of 3-note & 4-note chords are possible -- how to name them?
Helpful to have a canonical form:
        Starting from each note of chord, try adding others on
        top as closely as possible.
        Of these possible close voicings, choose the one whose top
        note is lowest.  If more than one have same top note, choose
        one with lowest 2nd-to-top note, etc.
Similarly, define "squishedness" of chords, where the canonical form is
the "most squished" voicing. (If chords have different numbers of notes,
define the one with fewer notes an being more squished.)

For a given number of notes N there are roughly (12 choose N)/12
canonical chords.  But some symmetric chords (e.g. aug, diminished)
have less than 12 versions of themselves.

# notes         # canonical chords
2               6
3		19
4               43 (40 asymmetric plus diminished, 7b5, 1 b2 b5 5)

Define an ordering of canonical chords with a given number of notes, as
follows: if one is less dissonant, it is lower.  If they're
equally dissonant, the _less_ squished one is lower.

Define canonical chord numbers starting from zero, where zero is
the least, etc. 

*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

double sin(),cos(),sqrt(),fabs();
#define PI 3.141592653589793

#define NEED_MAC_STUFF 0

#include "ear_prototypes.h"
#include "ear_defines.h"

#define MAX_NOTES_IN_CHORD 20

typedef int CHORD[MAX_NOTES_IN_CHORD];

#define ENUMERATION_STUFF 0

//----  The following lists were generated by enumerate_chords().
//	They give the quick labels of the canonical chords in order.
//	These can be accessed via quick_label_to_canonical_chord_number().

int n_canonical_chords[] = {-1,1,6,19,43};

//--- three-note chords:
/* total chords: 19 out of 19 desired */
unsigned long  enumerated_three_note_chords[] = {
	 116, /* dissonance=1,  notes= 0  4  7   M */
	 115, /* dissonance=1,  notes= 0  3  7   m */
	  99, /* dissonance=2,  notes= 0  3  6   dim */
	  98, /* dissonance=3,  notes= 0  2  6   7 (no 5) */
	  83, /* dissonance=3,  notes= 0  3  5   7 (no 3) */
	  82, /* dissonance=3,  notes= 0  2  5   m7 (no 5) */
	 114, /* dissonance=4,  notes= 0  2  7   sus4 */
	  84, /* dissonance=4,  notes= 0  4  5   maj7 (no 3) */
	  81, /* dissonance=4,  notes= 0  1  5   maj7 (no 5) */
	 132, /* dissonance=5,  notes= 0  4  8   + */
	 100, /* dissonance=5,  notes= 0  4  6   b5 */
	  67, /* dissonance=5,  notes= 0  3  4   m add #11 (no 1) */
	  66, /* dissonance=5,  notes= 0  2  4   M add 9 (no 5) */
	  65, /* dissonance=5,  notes= 0  1  4   m maj 7 (no 5) */
	 101, /* dissonance=6,  notes= 0  5  6   M add b9 (no 3) */
	  97, /* dissonance=6,  notes= 0  1  6   M add #11 (no 3) */
	  50, /* dissonance=6,  notes= 0  2  3   m add 9 (no 5) */
	  49, /* dissonance=6,  notes= 0  1  3   M add 11 (no 1) */
	  33 /* dissonance=8,  notes= 0  1  2   sus4 add #11 (no 1) */
};

//--- four-note chords:
/* total chords: 43 out of 43 desired */
unsigned long enumerated_four_note_chords[] = {
	2147, /* dissonance=1,  notes= 0  3  6  8   7 */
	2131, /* dissonance=2,  notes= 0  3  5  8   m7 */
	2129, /* dissonance=3,  notes= 0  1  5  8   maj7 */
	1858, /* dissonance=3,  notes= 0  2  4  7   M add 9 */
	2403, /* dissonance=4,  notes= 0  3  6  9   dim7 */
	2130, /* dissonance=4,  notes= 0  2  5  8   half-dim */
	2114, /* dissonance=4,  notes= 0  2  4  8   7+5 */
	1875, /* dissonance=4,  notes= 0  3  5  7   m add 11 */
	1874, /* dissonance=4,  notes= 0  2  5  7   7sus4 */
	2146, /* dissonance=5,  notes= 0  2  6  8   7b5 */
	2113, /* dissonance=5,  notes= 0  1  4  8   m maj 7 */
	1876, /* dissonance=5,  notes= 0  4  5  7   M add 11 */
	1842, /* dissonance=5,  notes= 0  2  3  7   m add 9 */
	1602, /* dissonance=5,  notes= 0  2  4  6   7 add 9 (no 5) */
	1892, /* dissonance=6,  notes= 0  4  6  7   M add #11 */
	1891, /* dissonance=6,  notes= 0  3  6  7   m add #11 */
	1841, /* dissonance=6,  notes= 0  1  3  7   m add b9 */
	1619, /* dissonance=6,  notes= 0  3  5  6   dim add 11 */
	1346, /* dissonance=6,  notes= 0  2  4  5   m7 add 9 (no 5) */
	1330, /* dissonance=6,  notes= 0  2  3  5   7 add 6 (no 3) */
	1329, /* dissonance=6,  notes= 0  1  3  5   maj7 add 9 (no 5) */
	2115, /* dissonance=7,  notes= 0  3  4  8   Mb6 */
	1890, /* dissonance=7,  notes= 0  2  6  7   7 add 11 (no 5) */
	1859, /* dissonance=7,  notes= 0  3  4  7   M add #9 */
	1857, /* dissonance=7,  notes= 0  1  4  7   M add b9 */
	1618, /* dissonance=7,  notes= 0  2  5  6   7 add #9 (no 5) */
	1617, /* dissonance=7,  notes= 0  1  5  6   maj7 add 11 (no 5) */
	1073, /* dissonance=7,  notes= 0  1  3  4   m maj 7 add 9 (no 5) */
	1873, /* dissonance=8,  notes= 0  1  5  7   b5 maj7 */
	1603, /* dissonance=8,  notes= 0  3  4  6   b5 add #9 */
	1601, /* dissonance=8,  notes= 0  1  4  6   b5 add b9 */
	1586, /* dissonance=8,  notes= 0  2  3  6   dim add 9 */
	1585, /* dissonance=8,  notes= 0  1  3  6   dim add b9 */
	1345, /* dissonance=8,  notes= 0  1  4  5   maj7 add #9 (no 5) */
	1347, /* dissonance=9,  notes= 0  3  4  5   7 maj 7 (no 3) */
	1313, /* dissonance=9,  notes= 0  1  2  5   m7 maj 7 (no 5) */
	1889, /* dissonance=10,  notes= 0  1  6  7   M add b9 add #11 (no 3) */
	1825, /* dissonance=10,  notes= 0  1  2  7   sus4 add #11 */
	1620, /* dissonance=10,  notes= 0  4  5  6   b5 add 11 */
	1569, /* dissonance=10,  notes= 0  1  2  6   7 maj 7 (no 5) */
	1074, /* dissonance=10,  notes= 0  2  3  4   m add 11 add #11 (no 1) */
	1057, /* dissonance=10,  notes= 0  1  2  4   M add b9 add 9 (no 5) */
	 801 /* dissonance=11,  notes= 0  1  2  3   m add b9 add 9 (no 5) */
};

unsigned long *all_enumerations[] = {
	(unsigned long *) 0,(unsigned long *) 0,(unsigned long *) 0,
	enumerated_three_note_chords,enumerated_four_note_chords};

	//-- returns -1 on error (bad value of n_notes or no such quick label)
	int
quick_label_to_canonical_chord_number(unsigned long quick_label,int n_notes)
	{
	  unsigned long *the_list;
	  int i,n;
	  switch(n_notes) {
	    case 3:
	      the_list = enumerated_three_note_chords;
	      break;
	    case 4:
	      the_list = enumerated_four_note_chords;
	      break;
	    default:
	      return -1;
	  }
	  n = n_canonical_chords[n_notes];
	  for (i=0; i<n; i++) {
	    if (the_list[i]==quick_label) return i;
	  }
	  return -1;
	}

	//-- Brings upper voices down as close as possible to bass.
	void
make_close_voicing(int *close_voicing,int *original_voicing,int n)
	{
	      int i;
	      copy_chord(close_voicing,original_voicing,n);
	      sort_int_list(close_voicing,n);
	      for (i=1; i<n; i++) {
	        while(close_voicing[i]-12>close_voicing[0])
	          close_voicing[i] -= 12;
	      }
	      sort_int_list(close_voicing,n);
	}

        /* see description at top of program
           returns 0 normally, -1 on error.
           Output is always sorted in ascending order, and
           1st note is always zero. */
	int
make_canonical_voicing(int *canonical_voicing,int *chord,int n_notes)
        {
          int i,j,k;
          int best_so_far[12]; /* max # notes is 12 */
          int this_one[12];
          int how_it_compares,any_swaps,temp;
          if (n_notes<1 || n_notes>12) return -1;
          if (n_notes==1) {
            canonical_voicing[0] = 0;
            return 0;
          }
          /* Check for notes that are the same. */
          for (i=1; i<n_notes; i++) {
            for (j=0; j<i; j++) {
              if ((chord[i]-chord[j])%12==0) return -1;
            }
          }
          for (i=0; i<n_notes; i++) {
            this_one[0] = chord[i];
            k = 1;
            for (j=0; j<n_notes; j++) {
              if (j!=i) {
                if (chord[j]==chord[i]) return -1; /* checked above, but make sure */
                this_one[k] = chord[j];
                while (this_one[k]>this_one[0])
                  this_one[k] -= 12;
                while (this_one[k]<this_one[0])
                  this_one[k] += 12;
                k++;
              } /* end if j!=i */
            } /* end for(j) */
            /* Now do bubble sort to put notes in ascending order: */
            do {
              any_swaps = 0;
              for (j=1; j<n_notes-1; j++) {
                if (this_one[j]>this_one[j+1]) {
                  temp = this_one[j];
                  this_one[j] = this_one[j+1];
                  this_one[j+1] = temp;
                  any_swaps = 1;
                }
              }
            } while(any_swaps);
            if (i==0) { /* make this the best so far, by default */
              how_it_compares = 1;
            }
            else { /* i!=0 */
              how_it_compares = compare_squishedness(this_one,best_so_far,n_notes);
            } /* end else, in code for deciding if this is an improvement */
            if (how_it_compares==1) {
              for (j=0; j<n_notes; j++) {
                best_so_far[j] = this_one[j];
              }
            }
          } /* end for(i), i.e. loop over possible bottom notes for voicings */
          for (j=0; j<n_notes; j++) {
            canonical_voicing[j] = best_so_far[j]-best_so_far[0];
          }
          return 0;
        }

        /* Returns 0 if chords a and b are equally squished,
           1 if a is more squished, -1 if a is less squished. */
	int
compare_squishedness(int *a,int *b,int n_notes)
        {
          int j;
          for (j=n_notes-1; j>0; j--) {
            if (a[j]-a[0]<b[j]-b[0])
              return 1;
            if (a[j]-a[0]>b[j]-b[0])
              return -1;
          }
          return 0;
        }

	int
same_canonical_form(int *a,int na,int *b,int nb)
	{
	  return (na==nb) && compare_chords(a,b,na)==0;
	}

        /* Returns 0 if chords a and b have same canonical form.
           Returns 1 if a is greater than b (i.e. a is more
             dissonant, or they're equally dissonant but a
             is less squished).
           Returns -1 is a is less then b.
           Both chords are converted to canonical form from the
           start. */
	int
compare_chords(int *a,int *b,int n_notes)
        {
          int canonical_a[12],canonical_b[12],diss_a,diss_b;
          make_canonical_voicing(canonical_a,a,n_notes);
          make_canonical_voicing(canonical_b,b,n_notes);
          if (identical_chords(canonical_a,canonical_b,n_notes))
            return 0;
          diss_a = dissonance(canonical_a,n_notes);
          diss_b = dissonance(canonical_b,n_notes);
          if (diss_a>diss_b) return 1;
          if (diss_a<diss_b) return -1;
          if (compare_squishedness(canonical_a,canonical_b,n_notes)==1)
            return 1;
          else
            return -1;
        }

        /* Identical, not just having same canonical form. */
	int
identical_chords(int *a,int *b,int n_notes)
        {
          int i;
          for (i=0; i<n_notes; i++) {
            if (a[i]!=b[i]) return 0;
          }
          return 1;
        }

	/*
		+1	to start with
		+1	for each 2, b5, or b7
		+2	for each b2 or M7
		+2	contains an augmented triad
		+2	doesn't contain any m or M triad
		..plus additional adjustments for special cases.
	*/
	int
dissonance(int *chord,int n_notes)
        {
          int i,j,k,score,interval;
          int number_of_2_or_b5,number_of_b2;
          int contains_minor_or_major,contains_augmented;
          int canonical_voicing[12];
          int subset_of_3[3],quick_label;
          if (n_notes<2) return 0;
          score = 1;
          number_of_2_or_b5 = 0;
          number_of_b2 = 0;
          for (i=1; i<n_notes; i++) {
            for (j=0; j<i; j++) {
              interval = chord[i]-chord[j];
              while(interval<0) interval+=12;
              while(interval>11) interval-=12;
              if (interval>6) interval = 12-interval; /* convert 7th to 2d, etc. */
              if (interval==1) ++number_of_b2;
              if (interval==6 || interval==2) ++number_of_2_or_b5;
            }
          }
          score += number_of_2_or_b5 + 2*number_of_b2;
          /* Does it contain a m, M or aug triad? ... */
          contains_minor_or_major = 0;
          contains_augmented = 0;
          for (i=2; i<n_notes; i++) {
            subset_of_3[0] = chord[i];
            for (j=1; j<i; j++) {
              subset_of_3[1] = chord[j];
              for (k=0; k<j; k++) {
                subset_of_3[2] = chord[k];
                make_canonical_voicing(canonical_voicing,subset_of_3,3);
                if (canonical_voicing[1]==4 && canonical_voicing[2]==7)
                  contains_minor_or_major = 1;
                if (canonical_voicing[1]==3 && canonical_voicing[2]==7)
                  contains_minor_or_major = 1;
                if (canonical_voicing[1]==4 && canonical_voicing[2]==8)
                  contains_augmented = 1;
              } /* end for (k) */
            } /* end for (j) */
          } /* end for (i) */
          if (!contains_minor_or_major) score+=2;
          if (contains_augmented) score+=2;

	  //--shift of scale for different numbers of notes:
	  switch(n_notes) {
	    case 4:
	      score += 1;
	      break;
	  }
	
	  //--additional adjustments for special cases:          

          make_canonical_voicing(canonical_voicing,chord,n_notes);
          quick_label = canonical_voicing_to_quick_label(canonical_voicing,n_notes);
          
          if (n_notes==3) {
            switch (quick_label) {
              case 99: score -= 2; break; //--  dim
              case 83: score -= 1; break; //--  7 no 3
              case 82: score -= 1; break; //--  m7 no 5
              case 98: score -= 2; break; //--  7 no 5
              case 84: score -= 1; break; //--  maj 7 no 3
              case 81: score -= 1; break; //--  maj 7 no 5
            }
          }

          if (n_notes==4) {
            switch (quick_label) {
              case 2147: score -= 3; break; //--  7
              case 2131: score -= 1; break; //--  m7
              case 2403: score -= 2; break; //--  dim7
              case 2130: score -= 0; break; //--  half-dim
              case 2129: score -= 1; break; //--  maj7
              case 1858: score -= 1; break; //--  add 9
              case 2113: score -= 1; break; //--  m maj 7
              case 2114: score -= 5; break; //--  7+5
              case 1859: score += 3; break; //--  M add #9
              case 1874: score -= 2; break; //-- 7sus4 
              case 2146: score -= 3; break; //-- 7b5
              case 1346: score -= 2; break; //--  m7 add 9 (no 5)
              case 1329: score -= 2 ; break; //--  maj7 add 9 (no 5)
              case 1617: score -= 2 ; break; //--  maj7 add 11 (no 5)
              case 1073: score -= 2 ; break; //--  m maj 7 add 9 (no 5)
              case 1876: score -= 0 ; break; //--  M add 11
              case 1857: score += 2; break; //--  M add b9
              case 1892: score -= 0; break; //--  M add #11
              case 1619: score -= 2; break; //--  dim add 11     (i.e 13 no 1)
              case 1841: score -= 0; break; //--  m add b9       (i.e. 13 no 5)
              case 1330: score -= 2; break; //--  7 add 6 (no 3) (i.e. 13 no 3)              
              case 1891: score += 1; break; //--  m add # 11
              case 801:  score -= 1; break; //--  m add b9 add 9 (no 5)
              case 2115:  score +=1 ; break; //--  M b6
              case 1890:  score -=1 ; break; //--  sus4 maj7 (i.e. 7 add 11, no 5)
              case 1618:  score -=1 ; break; //--  7 add #9 (no 5)
              case 1602:  score -=3 ; break; //--  b5 add 9 (i.e. 9 no 5)
            }
          }
          
          return score;
        }

	//-- if any two notes are a unison or differ only by octaves,
	//	eliminate the higher one
	void
remove_doublings(int *chord,int *n)
	{
	  int i,j,u,v,new_n;
	  int hit_list[MAX_CHORD_NOTES];
	  for (i=0; i<*n; i++) {
	    hit_list[i] = 0;
	  }
	  for (i=1; i<*n; i++) {
	    u = make_0_to_11(chord[i]);
	    for (j=0; j<i; j++) {
	      v = make_0_to_11(chord[j]);
	      if (u==v) {
	        if (chord[i]>=chord[j])
	          hit_list[i]=1;
	        else
	          hit_list[j]=1;
	      }
	    }
	  }
	  j=0;
	  new_n = 0;
	  for (i=0; i<*n; i++) {
	    if (!hit_list[i]) {
	      chord[j]=chord[i];
	      ++j;
	      ++new_n;
	    }
	  }
	  *n = new_n;
	}

	void
copy_chord(int *a,int *b,int n)
	{
	  int i;
	  for (i=0; i<n; i++)
	    a[i] = b[i];
	}
	
	void
swap_chords(int *a,int *b,int n)
	{
	  int i;
	  int temp[MAX_NOTES_IN_CHORD];
	  copy_chord(temp,a,n);
	  copy_chord(a,b,n);
	  copy_chord(b,temp,n);
	}

	int
make_0_to_11(int n)
	{
	  int u;
	  u = n;
	  while (u<0) u+=12;
	  while (u>11) u-=12;
	  return u;
	}

	int
inversion_of_interval(int n)
	{
	  if (n<0) n = -n;
	  while (n>0) n -= 12;
	  return -n;
	}

	void
sort_int_list(int *a,int n)
	{
	  int did_swap,i,temp;
	  do {
	    did_swap = 0;
	    for (i=0; i<n-1; i++) {
	      if (a[i]>a[i+1]) {
	        temp = a[i];
	        a[i] = a[i+1];
	        a[i+1] = temp;
	        did_swap = 1;
	      }
	    }
	  }while(did_swap);
	}

	void
make_chord(int *chord,int *n,int *chord_items,int n_items)
	{
	  int i,basic,x;
	  
	  *n = 0;
	  basic = find_basic_chord(chord_items,n_items);
	  //-- root
	  add_note_to_chord(chord,n,0);
	  //-- 3
	  x = what_is_3_of_chord(basic);
	  if (x != -1) add_note_to_chord(chord,n,x);
	  //-- 5
	  x = what_is_5_of_chord(basic);
	  if (x != -1) add_note_to_chord(chord,n,x);
	  //-- 7
	  x = what_is_7_of_chord(basic);
	  if (x != -1) add_note_to_chord(chord,n,x);
	  
	  //-- alterations:
	  for (i=0; i<n_items; i++) {
	    switch(chord_items[i]) {
	      case CHORD_FLAT9: 	add_note_to_chord(chord,n,1); break;
	      case CHORD_9: 		add_note_to_chord(chord,n,2); break;
	      case CHORD_SHARP9: 	add_note_to_chord(chord,n,3); break;
	      case CHORD_11: 		add_note_to_chord(chord,n,5); break;
	      case CHORD_SHARP11: 	add_note_to_chord(chord,n,6); break;
	      case CHORD_FLAT6: 	add_note_to_chord(chord,n,8); break;
	      case CHORD_6: 		add_note_to_chord(chord,n,9); break;
	      case CHORD_ADD7: 		add_note_to_chord(chord,n,10); break;
	      case CHORD_ADDMAJ7: 	add_note_to_chord(chord,n,11); break;
	      
	      case CHORD_NO1: 
	        remove_note_from_chord(chord,n,0);
	      	break;
	      case CHORD_NO3: 
	        x = what_is_3_of_chord(basic);
	        if (x != -1) remove_note_from_chord(chord,n,x);
	      	break;
	      case CHORD_NO5: 
	        x = what_is_5_of_chord(basic);
	        if (x != -1) remove_note_from_chord(chord,n,x);
	      	break;
	    }
	  }
	}

	int
chord_item_to_interval(int item)
	{
	  switch (item) {
	      case CHORD_FLAT9: 	return 1;
	      case CHORD_9: 		return 2;
	      case CHORD_SHARP9: 	return 3;
	      case CHORD_11: 		return 5;
	      case CHORD_SHARP11: 	return 6;
	      case CHORD_FLAT6: 	return 8;
	      case CHORD_6: 		return 9;
	      case CHORD_ADD7: 		return 10;
	      case CHORD_ADDMAJ7: 	return 11;
	      default:			return -1;
	  }
	}

	int
chord_item_to_scale_position(int item)
	{
	  switch (item) {
	      case CHORD_FLAT9: 	return 9;
	      case CHORD_9: 		return 9;
	      case CHORD_SHARP9: 	return 9;
	      case CHORD_11: 		return 11;
	      case CHORD_SHARP11: 	return 11;
	      case CHORD_FLAT6: 	return 6;
	      case CHORD_6: 		return 6;
	      case CHORD_ADD7: 		return 7;
	      case CHORD_ADDMAJ7: 	return 7;
	      default:			return -1;
	  }
	}

	int
what_is_3_of_chord(int basic_chord)
	{
	  switch(basic_chord) {
	    case CHORD_MAJ_TRIAD: return 4;
	    case CHORD_MIN_TRIAD: return 3;
	    case CHORD_DIM_TRIAD: return 3;
	    case CHORD_AUG_TRIAD: return 4;
	    case CHORD_SUS_TRIAD: return 5;
	    case CHORD_FLAT5_TRIAD: return 4;

	    case CHORD_DOM7: return 4;
	    case CHORD_MIN7: return 3;
	    case CHORD_MAJ7: return 4;
	    case CHORD_DIM7: return 3;
	    case CHORD_HALF_DIM: return 3;
	    default: return -1;
	  }
	}

	int
what_is_5_of_chord(int basic_chord)
	{
	  switch(basic_chord) {
	    case CHORD_MAJ_TRIAD: return 7;
	    case CHORD_MIN_TRIAD: return 7;
	    case CHORD_DIM_TRIAD: return 6;
	    case CHORD_AUG_TRIAD: return 8;
	    case CHORD_SUS_TRIAD: return 7;
	    case CHORD_FLAT5_TRIAD: return 6;

	    case CHORD_DOM7: return 7;
	    case CHORD_MIN7: return 7;
	    case CHORD_MAJ7: return 7;
	    case CHORD_DIM7: return 6;
	    case CHORD_HALF_DIM: return 6;
	    default: return -1;
	  }
	}

	int
what_is_7_of_chord(int basic_chord)
	{
	  switch(basic_chord) {
	    case CHORD_MAJ_TRIAD: return -1;
	    case CHORD_MIN_TRIAD: return -1;
	    case CHORD_DIM_TRIAD: return -1;
	    case CHORD_AUG_TRIAD: return -1;
	    case CHORD_SUS_TRIAD: return -1;
	    case CHORD_FLAT5_TRIAD: return -1;

	    case CHORD_DOM7: return 10;
	    case CHORD_MIN7: return 10;
	    case CHORD_MAJ7: return 11;
	    case CHORD_DIM7: return 9;
	    case CHORD_HALF_DIM: return 10;
	    default: return -1;
	  }
	}

	int
find_basic_chord(int *items,int n)
	{
	  int i,x;
	  for (i=0; i<n; i++) {
	    x = items[i];
	    if (x>=LO_TRIAD     && x<=HI_TRIAD) return x;
	    if (x>=LO_7TH_CHORD && x<=HI_7TH_CHORD) return x;
	  }
	  return -1;
	}

	void
add_to_list(int *list,int *n,int what)
	{
	  add_note_to_chord(list,n,what);
	}

	void
remove_from_list(int *list,int *n,int what)
	{
	  remove_note_from_chord(list,n,what);
	}

	void
add_note_to_chord(int *chord,int *n,int note)
	{
	  if (is_in_list(chord,*n,note)) return;
	  chord[*n] = note;
	  ++*n;
	}

	void
remove_note_from_chord(int *chord,int *n,int note)
	{
	  int i,j;
	  for (i=0; i<*n; i++) {
	    if (chord[i]==note) {
	      for (j=i; j<=*n-2; j++) {
	        chord[j] = chord[j+1];
	      }
	      --*n;
	    }
	  }
	}

	int
lists_are_same_except_for_order(int n1,int *l1,int n2,int *l2)
	{
	  int i;
	  if (n1!=n2) return 0;
	  for (i=0; i<n1; i++) {
	    if (!is_in_list(l2,n2,l1[i])) return 0;
	  }
	  return 1;
	}

	int
is_in_list(int *list,int n,int what)
	{
	  int i;
	  for (i=0; i<n; i++) {
	    if (list[i]==what) return 1;
	  }
	  return 0;
	}

	int
where_in_list(int *list,int n,int what)
	{
	  int i;
	  for (i=0; i<n; i++) {
	    if (list[i]==what) return i;
	  }
	  return -1;
	}

	int
how_unusual_is_chord(int *items,int n)
	{
	  int i,x;
	  x = 2 * how_unusual_is_basic_chord(find_basic_chord(items,n));
	  for (i=0; i<n; i++) {
	    if (items[i]>HI_7TH_CHORD) {
	      x += how_unusual_is_added_tone(items[i]);
	    }
	  }
	  return x;
	}

	// note that this is supposed to be an increasing function of the
	// item number for both triads and 7th chords
	int
how_unusual_is_basic_chord(int basic_chord)
	{
	  switch(basic_chord) {
	      case CHORD_MAJ_TRIAD:
	        return 1;
	      case CHORD_MIN_TRIAD:
	        return 1;
	      case CHORD_DIM_TRIAD:
	        return 4;
	      case CHORD_AUG_TRIAD:
	        return 5;
	      case CHORD_SUS_TRIAD:
	        return 5;
	      case CHORD_FLAT5_TRIAD:
	        return 6;
	      case CHORD_DOM7:
	        return 1;
	      case CHORD_MIN7:
	        return 2;
	      case CHORD_MAJ7:
	        return 3;
	      case CHORD_DIM7:
	        return 4;
	      case CHORD_HALF_DIM:
	        return 5;
	      default:
	        return 6;
	  }
	}

	int
how_unusual_is_added_tone(int tone)
	{
	  switch(tone) {
	      case CHORD_FLAT9:
	        return 5;
	      case CHORD_9:
	        return 2;
	      case CHORD_SHARP9:
	        return 6;
	      case CHORD_11:
	        return 3;
	      case CHORD_SHARP11:
	        return 4;
	      case CHORD_FLAT6:
	        return 6;
	      case CHORD_6:
	        return 1;
	      case CHORD_ADD7:
	        return 1;
	      case CHORD_ADDMAJ7:
	        return 2;
	      case CHORD_NO1:
	        return 1;
	      case CHORD_NO3:
	        return 4;
	      case CHORD_NO5:
	        return 1;
	      default:
	        return 6;
	  }
	}

	double
avg_chord(int *chord,int n)
	{
	  int i;
	  double z;
	  z = 0.;
	  for (i=0; i<n; i++) {
	    z += chord[i];
	  }
	  if (n>0)
	    return z/n;
	  else
	    return 0.;
	}



	void
describe_chord(char *s,int n,int *item)
	{
	  int i,basic;
	  char add[30];
	  int used_up[MAX_CHORD_ITEMS];
	  int special_case,has_dom7,has_maj7,has7,has_maj6,has_flat6,has6,
	  	have_omitted_tones,first_omitted_tone;
	  
	  basic = -1;
	  have_omitted_tones = 0;
	  
	  strcpy(s,"");
	  
	  for (i=0; i<n; i++) {
	    used_up[i] = 0;
	  }
	  for (i=0; i<n; i++) {
	    if (item[i]<LO_ADDED_TONE) {
	      basic=item[i];
	      used_up[i] = 1;
	    }
	    if (item[i]>=LO_OMITTED_TONE && item[i]<=HI_OMITTED_TONE) {
	      have_omitted_tones = 1;
	    }
	  }
	  
	  special_case = 0;
	  has_maj6 = is_in_list(item,n,CHORD_6);
	  has_flat6 = is_in_list(item,n,CHORD_FLAT6);
	  has6 =  has_maj6 || has_flat6;
	  has_dom7 = is_in_list(item,n,CHORD_ADD7) || 
	  		basic==CHORD_DOM7 || basic==CHORD_MIN7 ||
	  		basic==CHORD_HALF_DIM;
	  has_maj7 = is_in_list(item,n,CHORD_ADDMAJ7) || 
	  		basic==CHORD_MAJ7;
	  has7 = has_dom7 || has_maj7;
	  
	    //--- cases where the basic chord is a triad and they add a 7th,
	    //		which means the basic chord is really one of the basic
	    //		7th chords:
	    if (has_dom7 && basic==CHORD_MAJ_TRIAD) {
	      basic = CHORD_DOM7;
	      if (is_in_list(item,n,CHORD_ADD7))
	        used_up[where_in_list(item,n,CHORD_ADD7)] = 1;
	    }
	    if (has_dom7 && basic==CHORD_MIN_TRIAD) {
	      basic = CHORD_MIN7;
	      if (is_in_list(item,n,CHORD_ADD7))
	        used_up[where_in_list(item,n,CHORD_ADD7)] = 1;
	    }
	    if (has_dom7 && basic==CHORD_DIM_TRIAD) {
	      basic = CHORD_HALF_DIM;
	      if (is_in_list(item,n,CHORD_ADD7))
	        used_up[where_in_list(item,n,CHORD_ADD7)] = 1;
	    }
	    if (has_maj7 && basic==CHORD_MAJ_TRIAD) {
	      basic = CHORD_MAJ7;
	      if (is_in_list(item,n,CHORD_ADDMAJ7))
	        used_up[where_in_list(item,n,CHORD_ADDMAJ7)] = 1;
	    }
	    if (has_maj6 && basic==CHORD_DIM_TRIAD) {
	      basic = CHORD_DIM7;
	      if (is_in_list(item,n,CHORD_6))
	        used_up[where_in_list(item,n,CHORD_6)] = 1;
	    }
	  
	    //-- cases where the basic chord is one of the standard 7th
	    //		chords, and they add a redundant 7
	    if (is_in_list(item,n,CHORD_ADD7) && (basic==CHORD_DOM7
	    		|| basic==CHORD_MIN7 || basic==CHORD_HALF_DIM))
	      used_up[where_in_list(item,n,CHORD_ADD7)] = 1;
	    if (is_in_list(item,n,CHORD_ADDMAJ7) && basic==CHORD_MAJ7)
	      used_up[where_in_list(item,n,CHORD_ADDMAJ7)] = 1;
	    if (is_in_list(item,n,CHORD_6) && basic==CHORD_DIM7)
	      used_up[where_in_list(item,n,CHORD_6)] = 1;
	    
	    //-- other redundant added tones:
	    if (is_in_list(item,n,CHORD_11) && basic==CHORD_SUS_TRIAD)
	      used_up[where_in_list(item,n,CHORD_11)] = 1;
	    if (is_in_list(item,n,CHORD_FLAT6) && basic==CHORD_AUG_TRIAD)
	      used_up[where_in_list(item,n,CHORD_FLAT6)] = 1;
	    if (is_in_list(item,n,CHORD_SHARP11) && (basic==CHORD_FLAT5_TRIAD
	    	|| basic==CHORD_DIM_TRIAD || basic==CHORD_DIM7
	    	|| basic==CHORD_HALF_DIM) )
	      used_up[where_in_list(item,n,CHORD_SHARP11)] = 1;
	    if (is_in_list(item,n,CHORD_SHARP9) && (basic==CHORD_MIN_TRIAD
	    	|| basic==CHORD_DIM_TRIAD || basic==CHORD_MIN7
	    	|| basic==CHORD_DIM7 || basic==CHORD_HALF_DIM) )
	      used_up[where_in_list(item,n,CHORD_SHARP9)] = 1;
	      
	  if (basic==CHORD_MAJ_TRIAD && n==1) {
	    strcpy(s,"M");
	    special_case = 1;
	  }
	  if (!special_case && has_maj6 && !has7) {
	    if (basic==CHORD_MAJ_TRIAD || basic==CHORD_MIN_TRIAD
	    	|| basic==CHORD_AUG_TRIAD || basic==CHORD_SUS_TRIAD
	    	|| basic==CHORD_FLAT5_TRIAD) {
	      if (basic==CHORD_MAJ_TRIAD) {
	        strcpy(s,"6");
	      }
	      else {
	        describe_basic_chord(s,basic);
	        strcat(s,"6");
	      }
	      if (is_in_list(item,n,CHORD_6))
	        used_up[where_in_list(item,n,CHORD_6)] = 1;
	      special_case = 1;
	    }
	  }
	  if (!special_case && has_flat6 && !has7) {
	    if (basic==CHORD_MAJ_TRIAD || basic==CHORD_MIN_TRIAD
	    	|| basic==CHORD_DIM_TRIAD || basic==CHORD_SUS_TRIAD
	    	|| basic==CHORD_FLAT5_TRIAD) {
	      describe_basic_chord(s,basic);
	      strcat(s,"b6");
	      if (is_in_list(item,n,CHORD_FLAT6))
	        used_up[where_in_list(item,n,CHORD_FLAT6)] = 1;
	      special_case = 1;
	    }
	  }
	  if (!special_case && has_dom7) {
	    switch(basic) {
	      case CHORD_AUG_TRIAD: strcpy(s,"7+5"); special_case = 1; break;
	      case CHORD_SUS_TRIAD: strcpy(s,"7sus4"); special_case = 1; break;
	      case CHORD_FLAT5_TRIAD: strcpy(s,"7b5"); special_case = 1; break;
	    }
	    if (special_case && is_in_list(item,n,CHORD_ADD7))
	      used_up[where_in_list(item,n,CHORD_ADD7)] = 1;
	  }
	  if (!special_case && has_maj7) {
	    switch(basic) {
	      case CHORD_AUG_TRIAD: strcpy(s,"+5 maj7"); special_case = 1; break;
	      case CHORD_SUS_TRIAD: strcpy(s,"sus4 maj7"); special_case = 1; break;
	      case CHORD_FLAT5_TRIAD: strcpy(s,"b5 maj7"); special_case = 1; break;
	    }
	    if (special_case && is_in_list(item,n,CHORD_ADDMAJ7))
	      used_up[where_in_list(item,n,CHORD_ADDMAJ7)] = 1;
	  }
	  //-- 9th, 11th and 13th chords:
	  //		Note: we don't use this notation for b9, #11, etc.
	  if (!special_case && is_in_list(item,n,CHORD_9) 
	  	&& (basic==CHORD_DOM7 || basic==CHORD_MIN7 || basic==CHORD_MAJ7)) {
	    special_case = 1;
	    switch(basic) {
	      case CHORD_DOM7: strcpy(s,""); break;
	      case CHORD_MIN7: strcpy(s,"m"); break;
	      case CHORD_MAJ7: strcpy(s,"maj"); break;
	    }
	    used_up[where_in_list(item,n,CHORD_9)] = 1;
	    if (!is_in_list(item,n,CHORD_11)) {
	      strcat(s,"9");
	    }
	    else {
	      used_up[where_in_list(item,n,CHORD_11)] = 1;
	      if (!is_in_list(item,n,CHORD_6)) {
	        strcat(s,"11");
	      }
	      else {
	        strcat(s,"13");
	        used_up[where_in_list(item,n,CHORD_6)] = 1;
	      }
	    }
	  }
	  
	  if (!special_case) {
	    if (basic!= -1)
	      describe_basic_chord(s,basic);
	    else
	      strcpy(s,"");
	  }
	  
	  //--- now do everything but omitted tones:
	  for (i=0; i<n; i++) {
	    if (!used_up[i] && !(item[i]>=LO_OMITTED_TONE && item[i]<=HI_OMITTED_TONE)) {
	      describe_added_tone(add,item[i]);
	      strcat(s," ");
	      strcat(s,add);
	      used_up[i] = 1;
	    }
	  }
	  
	  //--- put omitted tones at the end, in parentheses:
	  if (have_omitted_tones) {
	    first_omitted_tone = 1;
	    strcat(s," (");
	    for (i=0; i<n; i++) {
	      if (!used_up[i]) {
	        describe_added_tone(add,item[i]);
	        if (!first_omitted_tone) strcat(s," ");
	        strcat(s,add);
	        used_up[i] = 1;
	        first_omitted_tone = 0;
	      }
	    }
	    strcat(s,")");
	  }
	  
	  
	}

	void
describe_basic_chord(char *s,int basic)
	{
	  switch(basic) {
	    case CHORD_MAJ_TRIAD: strcpy(s,"M"); break;
	    case CHORD_MIN_TRIAD: strcpy(s,"m"); break;
	    case CHORD_DIM_TRIAD: strcpy(s,"dim"); break;
	    case CHORD_AUG_TRIAD: strcpy(s,"+"); break;
	    case CHORD_SUS_TRIAD: strcpy(s,"sus4"); break;
	    case CHORD_FLAT5_TRIAD: strcpy(s,"b5"); break;
	    case CHORD_DOM7: strcpy(s,"7"); break;
	    case CHORD_MIN7: strcpy(s,"m7"); break;
	    case CHORD_MAJ7: strcpy(s,"maj7"); break;
	    case CHORD_DIM7: strcpy(s,"dim7"); break;
	    case CHORD_HALF_DIM: strcpy(s,"half-dim"); break;
	    default: strcpy(s,""); break;
	  }
	}

	void
describe_added_tone(char *s,int what)
	{
	  switch(what) {
	    case CHORD_FLAT9: strcpy(s,"add b9"); break;
	    case CHORD_9: strcpy(s,"add 9"); break;
	    case CHORD_SHARP9: strcpy(s,"add #9"); break;
	    case CHORD_11: strcpy(s,"add 11"); break;
	    case CHORD_SHARP11: strcpy(s,"add #11"); break;
	    case CHORD_FLAT6: strcpy(s,"add b6"); break;
	    case CHORD_6: strcpy(s,"add 6"); break;
	    case CHORD_ADD7: strcpy(s,"7"); break;
	    case CHORD_ADDMAJ7: strcpy(s,"maj 7"); break;
	    case CHORD_NO1: strcpy(s,"no 1"); break;
	    case CHORD_NO3: strcpy(s,"no 3"); break;
	    case CHORD_NO5: strcpy(s,"no 5"); break;
	    default: strcpy(s,""); break;
	  }
	}

	//--Search for an alternative description of a chord that is simpler.
	//	Descriptions containing fewer items are always considered
	//	simpler than ones containing more.
	//	Descriptions containing the same number of items are
	//	judged based on the result of how_unusual_is_chord().
	void
simplify_chord(int *items,int *n_items)
	{
	  int chord[MAX_CHORD_NOTES],n;
	  int new_items[MAX_CHORD_NOTES],n_new_items;
	  make_chord(chord,&n,items,*n_items);
	  notes_to_items(new_items,&n_new_items,chord,n);
	  if (n_new_items==0) return; //-- shouldn't really happen
	  if (!items_describe_same_chord(items,*n_items,new_items,n_new_items)) {
	  	//-- this also shouldn't happen
	    return;
	  }
	  copy_chord(items,new_items,n_new_items);
	  *n_items = n_new_items;
	}

	int
items_describe_same_chord(int *items1,int nitems1,int *items2,int nitems2)
	{
	  int chord1[MAX_CHORD_NOTES],n_notes1;
	  int chord2[MAX_CHORD_NOTES],n_notes2;
	  make_chord(chord1,&n_notes1,items1,nitems1);
	  make_chord(chord2,&n_notes2,items2,nitems2);
	  return same_canonical_form(chord1,n_notes1,chord2,n_notes2);
	}

	void
notes_to_items(int *items,int *n_items,int *chord,int n_notes)
	{
	  int ch[MAX_CHORD_NOTES]; //-- my copy, in which I'll make all notes
	  int n;		   // lie within range 0-11, eliminate doublings
	  int root,best_root,fewest_items,least_unusual,unusualness;
	  int can[MAX_CHORD_NOTES];
	  unsigned long quick_label;
	  copy_chord(ch,chord,n_notes);
	  n = n_notes;
	  remove_doublings(ch,&n);
	  make_canonical_voicing(can,ch,n);
	  quick_label = canonical_voicing_to_quick_label(can,n);
	  
	  //--- special cases:
	  if (n==4) {
	    switch(quick_label) {
	      case 1602:
	        *n_items = 3;
	        items[0] = CHORD_DOM7;
	        items[1] = CHORD_9;
	        items[2] = CHORD_NO5;
	        return;
	      case 1890:
	        *n_items = 3;
	        items[0] = CHORD_DOM7;
	        items[1] = CHORD_11;
	        items[2] = CHORD_NO5;
	        return;
	    }
	  }
	  
	  best_root = -1;
	  fewest_items = 999;
	  least_unusual = 999;
	  for (root=0; root<=11; root++) { //-- try to analyze the chord by taking
	  				// every possible note as its root
	    analyze_chord_based_on_this_root(items,n_items,ch,n,root);
	    if (*n_items!=0) { //-- it is possible to analyze it based on this root
	      unusualness = how_unusual_is_chord(items,*n_items);
	      if (*n_items<fewest_items
	      	     || (*n_items==fewest_items && unusualness<least_unusual) ) {
	      	best_root = root;
	        fewest_items = *n_items;
	        least_unusual = unusualness;
	      }
	    }
	  }
	  if (best_root == -1) { //-- we failed
	    *n_items = 0;
	  }
	  else {
	    analyze_chord_based_on_this_root(items,n_items,ch,n,best_root);
	  }
	}

	void
spell_note_as_part_of_chord(
	int *err,			// == 0 normally
	int *note_letter,		// C=0, D=1, ... B=6
	int *accidental,		// natural=0, flat=-1, sharp=1, double flat=-2, ...
	int interval_above_root,	// note to spell, in units of semitones
	int root_letter,
	int root_accidental,
	int *items,
	int n_items,
	int spell_dim7_enharmonically)
	{
	  int i,mister_interval,basic,the3,the5,the7,got_it,scale_position;
	  
	  basic = find_basic_chord(items,n_items);

	  the3 = what_is_3_of_chord(basic);
	  the5 = what_is_5_of_chord(basic);
	  the7 = what_is_7_of_chord(basic);
	  
	  got_it = 0;
	  
	  if (make_0_to_11(interval_above_root)==0) {
	    scale_position = 1;
	    got_it = 1;
	  }
	  if (make_0_to_11(the3-interval_above_root)==0) {
	    if (the3<5)
	      scale_position = 3;
	    else
	      scale_position = 4; //-- "the3" is really the 4 of a sus4 triad
	    got_it = 1;
	  }
	 if (!got_it && make_0_to_11(the5-interval_above_root)==0) {
	    scale_position = 5;
	    got_it = 1;
	 }
	 if (!got_it && the7 != -1 && make_0_to_11(the7-interval_above_root)==0) {
	    if (the7>9 || !spell_dim7_enharmonically)
	      scale_position = 7;
	    else
	      scale_position = 6; //-- 7 of dim7 is really 6 (avoid double flats)
	    got_it = 1;
	 }
	 
	 if (!got_it) {
	   for (i=0; i<n_items && !got_it; i++) {
	     mister_interval = chord_item_to_interval(items[i]);
	     if (mister_interval != -1 
	     	    && make_0_to_11(mister_interval-interval_above_root)==0) {
	       scale_position = chord_item_to_scale_position(items[i]);
	       got_it = 1;
	     }
	   }
	 }
	 
	 if (!got_it) { //-- this should never happen
	   *err = -1;
	   return;
	 }

	 *note_letter = root_letter+scale_position-1;
	 while (*note_letter>6) *note_letter -= 7;
	 while (*note_letter<0) *note_letter += 7;

	 *accidental =   interval_above_root
	 		-(
	 		   letter_and_accidental_to_note(*note_letter,0)
	 		  -letter_and_accidental_to_note(root_letter,root_accidental)
	 		);
	 while (*accidental < -6) *accidental = *accidental + 12;
	 while (*accidental > 6)  *accidental = *accidental - 12;
	 

	 *err = 0;

	}

	int
letter_and_accidental_to_note(int letter,int accidental)
	{
	  int note;
	  switch(letter) {
	    case 0: note = 0;  break; // C
	    case 1: note = 2;  break; // D
	    case 2: note = 4;  break; // E
	    case 3: note = 5;  break; // F
	    case 4: note = 7;  break; // G
	    case 5: note = 9;  break; // A
	    case 6: note = 11; break; // B
	  }
	  return note + accidental;
	}

	// -- input chord should have no doublings
	// -- returns with *n_items==0 if it can't analyze it
	void
analyze_chord_based_on_this_root(int *items,int *n_items,int *raw_ch,int n,int root)
	{
	  int basic,best_basic,fewest_items,least_unusual,no_way_dude,
	  	the3,the5,the7,i,current_items[MAX_CHORD_NOTES],current_n_items,
	  	rel_note,unusualness,ch[MAX_CHORD_NOTES];
	  copy_chord(ch,raw_ch,n);
	  for (i=0; i<n; i++)
	    ch[i] = make_0_to_11(ch[i]);
	  best_basic = -1;
	  fewest_items = 999;
	  least_unusual = 999;
	  for (basic=LO_TRIAD; basic<=HI_7TH_CHORD; basic++) {
	    if (basic>HI_TRIAD && basic<LO_7TH_CHORD) basic=LO_7TH_CHORD;
	    //-- We can't analyze it this way if it would involve adding
	    //		a M3 or P5, or removing the 7th.
	    the3 = what_is_3_of_chord(basic);
	    the5 = what_is_5_of_chord(basic);
	    the7 = what_is_7_of_chord(basic);
	    no_way_dude = ( the3 != 4 && is_in_list(ch,n,make_0_to_11(root+4)) );
	    no_way_dude = no_way_dude ||
	    		  ( the5 != 7 && is_in_list(ch,n,make_0_to_11(root+7)) );
	    no_way_dude = no_way_dude ||
	    		  ( the7 != -1 && !is_in_list(ch,n,make_0_to_11(root+the7)) );
	    if (!no_way_dude) {
	      current_n_items = 1;
	      current_items[0] = basic;
	      //-- which notes of this basic chord would we have to omit?
	      if (!is_in_list(ch,n,make_0_to_11(root))) 
	        add_to_list(current_items,&current_n_items,CHORD_NO1);
	      if (!is_in_list(ch,n,make_0_to_11(root+the3)))
	        add_to_list(current_items,&current_n_items,CHORD_NO3);
	      if (!is_in_list(ch,n,make_0_to_11(root+the5)))
	        add_to_list(current_items,&current_n_items,CHORD_NO5);
	      //-- which notes would we have to add?
	      for (i=0; i<n; i++) {
	        rel_note = make_0_to_11(ch[i]-root);
	        if (	   rel_note!=0 
	        	&& rel_note!=the3
	        	&& rel_note!=the5
	        	&& (the7 == -1 || rel_note!=the7 ) )
	          add_to_list(current_items,&current_n_items,
	          	note_to_chord_item(rel_note));
	      }//-- end for i
	      unusualness = how_unusual_is_chord(current_items,current_n_items);
	      if (best_basic<0 || current_n_items<fewest_items
	      	  || ( current_n_items==fewest_items && unusualness<least_unusual)) {
	      	best_basic = basic;
	      	fewest_items = current_n_items;
	      	least_unusual = unusualness;
	      	copy_chord(items,current_items,current_n_items);
	      	*n_items = current_n_items;
	      }
	    }//-- end if we can analyze it using this basic chord
	  }//-- end loop over basic chords
	  if (best_basic<0) *n_items = 0;
	}

	int
note_to_chord_item(int note)
	{
	  switch(make_0_to_11(note)) {
	    case 1: return CHORD_FLAT9;
	    case 2: return CHORD_9;
	    case 3: return CHORD_SHARP9;
	    case 5: return CHORD_11;
	    case 6: return CHORD_SHARP11;
	    case 8: return CHORD_FLAT6;
	    case 9: return CHORD_6;
	    case 10: return CHORD_ADD7;
	    case 11: return CHORD_ADDMAJ7;
	    default: return -1;
	  }
	}

	unsigned long
notes_to_quick_label(int *notes,int n)
	{
	  int can[MAX_NOTES_IN_CHORD],not_doubled[MAX_NOTES_IN_CHORD],nn;
	  copy_chord(not_doubled,notes,n);
	  nn = n;
	  remove_doublings(not_doubled,&nn);
	  make_canonical_voicing(can,not_doubled,nn);
	  return canonical_voicing_to_quick_label(can,nn);
	}
	

	unsigned long
canonical_voicing_to_quick_label(int *notes,int n)
	{
	  unsigned long quick_label;
	  int i;
	  quick_label = 0;
	  for (i=n-1; i>=1; i--) {
	    quick_label = quick_label << 4;
	    quick_label = quick_label | notes[i];
	  }
	  return quick_label;
	}


	int
nifty_chord_number(int *original_notes,int original_n)
	{
	  int a,b,tot,i;
	  unsigned long quick_label,*the_list;
	  int notes[MAX_NOTES_IN_CHORD],n;
	  
	  copy_chord(notes,original_notes,original_n);
	  n = original_n;
	  
	  remove_doublings(notes,&n);
	  
	  switch(n) {
	    case 1: return 1;
	    case 2:
	      a = 1+make_0_to_11(notes[1]-notes[0]);
	      b = 1+make_0_to_11(notes[0]-notes[1]);
	      if (a<b) return a; else return b;
	    case 3:
	    case 4:
	      tot = n_canonical_chords[n];
	      quick_label = notes_to_quick_label(notes,n);
	      for (i=0; i<tot; i++) {
	        if (quick_label==all_enumerations[n][i]) return i+1;
	      }
	      return -1;
	    default:
	      return -1;
	  }
	}

	void
describe_accidental(char *s,int accidental)
	{
              switch(accidental) {
                case  1: strcpy(s,"#"); break;
                case -1: strcpy(s,"b"); break;
                case  2: strcpy(s,"x"); break;
                case -2: strcpy(s,"bb"); break;
                case  3: strcpy(s,"x#"); break;
                case -3: strcpy(s,"bbb"); break;
                default: strcpy(s,""); break;
              }
	}


#if ENUMERATION_STUFF
	void
enumerate_chords(int n_notes,int n_tries,int n_desired)
	{
	  static CHORD list[100];
	  static int used_up[100];
	  static int items[100];
	  static char description[100];
	  int n_chords,i,j,note,err,it_is_new,best,n_items;
	  CHORD this_chord,can;
	  FILE *f;
	  f = fopen("a.a","w");
	  if (f == (FILE *) 0) return;
	  n_chords = 0;
	  for (i=1; i<=n_tries && n_chords<n_desired; i++) {
	    this_chord[0] = 0;
	    for (j=1; j<n_notes; j++) {
	      do {
	        note = random_integer(11); //-- 1 to 11
	      } while (is_in_list(this_chord,j-1,note));
	      this_chord[j] = note;
	    }
	    err = make_canonical_voicing(can,this_chord,n_notes);
	    if (err==0) {
	      it_is_new = 1;
	      for (j=0; j<n_chords; j++) {
	        if (identical_chords(list[j],can,n_notes)) {
	          it_is_new = 0;
	          break;
		}
	      }
	      if (it_is_new) {
	        copy_chord(list[n_chords],can,n_notes);
	        ++n_chords;
	      }
	    }//--end if no err
	  }
	  for (i=0; i<n_chords; i++) {
	    used_up[i] = 0;
	  }
	  fprintf(f,"/* total chords: %d out of %d desired */\n",n_chords,n_desired);
	  for (i=0; i<n_chords; i++) {
	    best = -1;
	    for (j=0; j<n_chords; j++) {
	      if (!used_up[j] && 
	      		(best == -1 || compare_chords(list[j],list[best],n_notes)<0))
	        best = j;
	    }
	    if (best != -1) {
	      used_up[best] = 1;
	      notes_to_items(items,&n_items,list[best],n_notes);
	      describe_chord(description,n_items,items);
	      fprintf(f,"%4d", 
	           canonical_voicing_to_quick_label(list[best],n_notes));
	      if (i<n_chords-1) fprintf(f,",");
	      fprintf(f," /* dissonance=%d,  notes=",
	      	   dissonance(list[best],n_notes));
	      for (j=0; j<n_notes; j++) {
	        fprintf(f,"%2d ",list[best][j]);
	      }
	      fprintf(f,"  %s */\n",description);
	    }
	  }
	  fclose(f);
	}
#endif


