//
//	random_pmbdl.cp
//
//	random number generator a la Park/Miller, Bays/Durham, L'Ecuyer
//
//	Converted from the FORTRAN version in Numerical Recipes in Fortran,
//	Press, Teukolsky, Vetterling and Flannery, subroutine ran2, p. 272.
//	Should produce identical results, although I implemented it slightly
//	differently.  Should compile in C, if your compiler can handle
//	C++-style comments.  Make sure to typedef LONG_T correctly for
//	your machine (e.g. alphas have 64-bit longs).
//
//	Basic idea from Park and Miller:
//			I(j+1) = a * I(j) (mod m)
//		where a=7**5, m=2**31-1
//	Plus shuffling idea from Bays and Durham:
//		Make a table of 32 random numbers and shuffle randomly
//		to decide which to use next.  Gets rid of correlations.
//	Plus idea by L'Ecuyer to combine two sequences to make the
//		period much much longer.
//
//	Routines:
//		random_pmbdl_seed	sets the seed
//		random_pmbdl_real	returns double from 0 to 1
//		random_pmbdl_int32	returns 32-bit integer
//		random_pmbdl_int16	returns 16-bit integer
//		random_pmbdl_gaussian	returns double with Gaussian distn
//		random_pmbdl_basic	don't call directly
//	Note that if you really want a real, it's _more_ efficient
//	to call random_pmbdl_real, rather than calling random_pmbdl_int32
//	and dividing by 2**32.  This is because random_pmbdl_int32
//	has to call random_pmbdl_basic twice.
//	All the routines are eating numbers from the same sequence,
//	based on the same seed.

//----------- copy into your code: -------------------------------------

typedef long LONG_T;
typedef unsigned long ULONG_T;

double 	random_pmbdl_real();
ULONG_T random_pmbdl_int32();
ULONG_T random_pmbdl_int16();
void 	random_pmbdl_seed(ULONG_T);
double 	random_pmbdl_gaussian();

//------------------------------------------------------------------
LONG_T  random_pmbdl_basic(LONG_T,int);

#include <math.h>
extern double sqrt(double),log(double);

#define BUMP (-1)
	//...offset for C array indices as opposed to FORTRAN

#define IM1 ((LONG_T) 2147483563L)
		// = 2**31-85
#define IM2 ((LONG_T) 2147483399L)
#define IMM1 (IM1-1)
#define IA1 ((LONG_T) 40014L)
#define IA2 ((LONG_T) 40692L)
#define IQ1 ((LONG_T) 53668L)
#define IQ2 ((LONG_T) 52774L)
#define IR1 ((LONG_T) 12211L)
#define IR2 ((LONG_T) 3791L)
#define NTAB (32)
#define NDIV (1+IMM1/NTAB)
		// note that this is integer division, producing truncation
#define EPS (1.2e-7)
#define RNMX (1.-EPS)

	double
random_pmbdl_gaussian()
	{
	  double fac,rsq,v1,v2;
	  static int set=0;
	  static double gset;
	  if (set==0) {
	    do {
	      v1 = 2.*random_pmbdl_real()-1.;
	      v2 = 2.*random_pmbdl_real()-1.;
	      rsq = v1*v1+v2*v2;
	    } while (rsq>=1. || rsq==0.);
	    fac = sqrt(-2.*log(rsq)/rsq);
	    gset = v1*fac;
	    set = 1;
	    return v2*fac;
	  }
	  else {
	    set = 0;
	    return gset;
	  }
	}
	
	double
random_pmbdl_real()
	{
	  double z;
	  z = random_pmbdl_basic((LONG_T) 0,0);
	  z = z/IM1;
	  if (z>RNMX) z=RNMX;	// make it behave just like Numerical Recipes' version
	  return z;
	}
	
	ULONG_T
random_pmbdl_int32()
	{
	  
	  return random_pmbdl_int16()<<16 | random_pmbdl_int16();
	}
	
	ULONG_T
random_pmbdl_int16()
	{
	  return ((ULONG_T) 0xFFFFL) & ((ULONG_T) random_pmbdl_basic((LONG_T) 0,0)>>13);
	  	// We want to right-shift to avoid taking least-significant
	  	// bit, which are generally less reliable.  But the top couple
	  	// of bits are non-random because the maximum returned from
	  	// the basic routine is less than 2**32-1.
	}
	
	void
random_pmbdl_seed(ULONG_T seed)
		// make it ULONG_T for convenience of user, so they only ever deal
		// with ULONG_T, not LONG_T
	{
	  random_pmbdl_basic((LONG_T) seed,1);
	  	// possible sign extension won't kill us, since the basic routine
	  	// masks off the high bit anyway
	}

	// Returns a random number from 1 to 2**31-85
	// To initialize, call with initialize=1 and a seed (high bit ignored).
	// Seed is ignored if initialize=0.
	// This is different from Numerical Recipes' ran2, which uses
	// negative idum to initialize.
	// Although there is really no reason to use signed arithmetic for
	// such a routine, Numerical Recipes' ran2 uses signed arithmetic
	// because it's in FORTRAN, so I use signed arithmetic too.
	// The output is always positive.
	LONG_T
random_pmbdl_basic(LONG_T seed,int initialize)
	{
	  static LONG_T idum = 1; // just in case they call without initializing
	  static LONG_T idum2 = 123456789;
	  static LONG_T iy = 0;
	  static LONG_T iv[NTAB] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	  			     0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	  static LONG_T j,k;
	  if (initialize) {
	    idum = seed & ((LONG_T) 0x7FFFFFFFL);
	    if (idum==0) idum=1; //--0 as a seed always produces bad results
	    idum2 = idum;
	    //--- Do 8 warmups, then fill in shuffle table:
	    for (j=NTAB+8; j>=1; j--) {
	      k = idum/IQ1;
	      idum = IA1*(idum-k*IQ1)-k*IR1;
	      if (idum<0) idum += IM1;
	      if (j<=NTAB) iv[j+BUMP] = idum;
	    }//--end loop over j
	    iy = iv[1+BUMP];
	  }//--end if initializing
	  k = idum/IQ1;
	  idum = IA1*(idum-k*IQ1)-k*IR1; // compute idum=mod(IA1*idum,IM1) without
	  				 // overflows by Schrage's method
	  if (idum<0) idum += IM1;
	  k = idum2/IQ2;
	  idum2 = IA2*(idum2-k*IQ2)-k*IR2;
	  if (idum2<0) idum2 += IM2;
	  j = 1+iy/NDIV;		// make it in range from 1 to NTAB
	  iy = iv[j+BUMP]-idum2;
	  iv[j+BUMP] = idum;
	  if (iy<1) iy += IMM1;
	  return iy;
	}



